import yargs from 'yargs';
import { hideBin } from 'yargs/helpers';
import inquirer from 'inquirer';
import { createOpenAiClient } from './app';
import { OpenAi } from '@modal-sh/mio-ai';

export type Argv = Record<string, unknown>;

const main = (argv: Argv) => new Promise<number>((resolve) => {
  const aiClient = createOpenAiClient({
    apiKey: process.env.OPENAI_API_KEY as string,
    organizationId: process.env.OPENAI_ORGANIZATION_ID as string,
    apiVersion: OpenAi.ApiVersion.V1,
  });
  let result: Partial<OpenAi.ChatCompletion> | undefined;
  let done = false;

  aiClient.on<OpenAi.ChatCompletionChunkDataEvent>('data', (d) => {
    d.choices.forEach((c) => {
      if (!result) {
        result = {
          id: d.id,
          object: OpenAi.ChatCompletionDataEventObjectType.CHAT_COMPLETION,
          created: d.created,
          model: d.model,
        };
      }

      if (!Array.isArray(result?.choices)) {
        result.choices = [];
      }

      if (!result.choices[c.index]) {
        result.choices[c.index] = {
          message: { content: '' },
          index: c.index,
          finish_reason: c.finish_reason,
        };
      }

      if (result.choices[c.index].message) {
        if (c.delta.role) {
          (result.choices[c.index].message as Record<string, unknown>).role = c.delta.role;
        }

        if (c.delta.content) {
          (result.choices[c.index].message as Record<string, unknown>)
            .content += c.delta.content;
        }
      }

      if (c.finish_reason) {
        result.choices[c.index].finish_reason = c.finish_reason;
      }
    });
  });

  aiClient.on('end', () => {
    // resolve?.(result);
  });

  aiClient.on('error', (error: Error) => {
    done = true;
    resolve(-1);
  });

  inquirer.prompt([
    {
      type: 'input',
      message: 'Type your inquiry.',
    },
  ])
    .then((answers) => {
      console.log(answers);
      // aiClient.createChatCompletion({
      //   model: OpenAi.ChatCompletionModel.GPT_3_5_TURBO,
      //   messages: {
      //     role: OpenAi.MessageRole.USER,
      //     content: answers[0] as string,
      //   },
      // });
    })
    .catch(() => {
      resolve(-1);
    });

  resolve(0);
});

main(yargs(hideBin(process.argv)).argv as unknown as Argv)
  .then((result) => {
    process.exit(result);
  })
  .catch((error) => {
    console.error(error);
    process.exit(-1);
  });
